<?php
/**
* Razer Merchant Services Prestashop Plugin
*
* @package Payment Method
* @author Razer Merchant Services Technical Team <technical-sa@razer.com>
*
*/
use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_'))
    exit;

class Axaipay extends PaymentModule {
    private $_html = '';
	private $_postErrors = array();
    public function __construct() {
		$this->name = 'axaipay';
		$this->tab = 'payments_gateways';
		$this->version = '1.0.0';
		$this->author = 'Axai Digital Team';
		$this->author_uri = 'https://axaipay.freshdesk.com/support/solutions/folders/73000400936';
		$this->controllers = array('validation');
		$this->bout_valide = $this->l('Validate');        
		$this->currencies = true;
		$this->currencies_mode = 'checkbox';        
		$this->bootstrap = true;

		$config = Configuration::getMultiple(array('AXAIPAY_MERCHANT_ID', 'AXAIPAY_SIGNING_KEY', 'AXAIPAY_SANDBOX_MERCHANT_ID', 'AXAIPAY_SANDBOX_SIGNING_KEY', 'AXAIPAY_ENVIRONMENT'));
		if(isset($config['AXAIPAY_MERCHANT_ID']))
			$this->AXAIPAY_MERCHANT_ID = $config['AXAIPAY_MERCHANT_ID'];
        if(isset($config['AXAIPAY_SIGNING_KEY']))
			$this->AXAIPAY_SIGNING_KEY = $config['AXAIPAY_SIGNING_KEY'];
        if(isset($config['AXAIPAY_SANDBOX_MERCHANT_ID']))
			$this->AXAIPAY_SANDBOX_MERCHANT_ID = $config['AXAIPAY_SANDBOX_MERCHANT_ID'];
        if(isset($config['AXAIPAY_SANDBOX_SIGNING_KEY']))
			$this->AXAIPAY_SANDBOX_SIGNING_KEY = $config['AXAIPAY_SANDBOX_SIGNING_KEY'];
        if(isset($config['AXAIPAY_ENVIRONMENT']))
            $this->AXAIPAY_ENVIRONMENT = $config['AXAIPAY_ENVIRONMENT'];

		parent::__construct();
		$this->displayName = 'Axaipay';
		$this->description = $this->l('Malaysia Payment Gateway - Redefining Digital Payment Solution');
		$this->confirmUninstall = $this->l('Are you sure you want to delete your details ?');

		if(!count(Currency::checkPaymentCurrencies($this->id)))
				$this->warning = $this->l('No currency set for this module');
		if(!isset($this->AXAIPAY_MERCHANT_ID) || !isset($this->AXAIPAY_SIGNING_KEY) || !isset($this->AXAIPAY_SANDBOX_MERCHANT_ID) || !isset($this->AXAIPAY_SANDBOX_SIGNING_KEY))
				$this->warning = $this->l('Your Axaipay account must be set correctly');
		if(!isset($this->AXAIPAY_ENVIRONMENT))
				$this->warning = $this->l('This plugin required an environment type selected.');
	}

    /**
	 * Install the Razer Merchant Services module into prestashop
	 * 
	 * @return boolean
	 */
	public function install() {
		if (!parent::install() || !$this->registerHook('paymentOptions') || !$this->registerHook('paymentReturn') || !$this->registerHook('payment') || !$this->registerHook('header') )
			return false;
		else
			return true;
	}

    /**
	 * Uninstall the Razer Merchant Services module from prestashop
	 * 
	 * @return boolean
	 */
	public function uninstall() {
		if (!Configuration::deleteByName('AXAIPAY_MERCHANT_ID') || !Configuration::deleteByName('AXAIPAY_SIGNING_KEY') || !Configuration::deleteByName('AXAIPAY_SANDBOX_MERCHANT_ID') || !Configuration::deleteByName('AXAIPAY_SANDBOX_SIGNING_KEY') || !Configuration::deleteByName('AXAIPAY_ENVIRONMENT') || !parent::uninstall())
			return false;
		else
			return true;
	}

    /**
	 * Validate the form submited by Razer Merchant Services configuration setting
	 * 
	 */
	protected function _postValidation() {
		if (Tools::isSubmit('btnSubmit')) {
			if (!Tools::getValue('merchant_id'))
				$this->_postErrors[] = $this->l('Merchant ID is required');
			else if (!Tools::getValue('signing_key'))
				$this->_postErrors[] = $this->l('Signing Key is required.');
			else if (!Tools::getValue('sandbox_merchant_id'))
                $this->_postErrors[] = $this->l('Sandbox Merchant ID is required.');	
            else if (!Tools::getValue('sandbox_signing_key'))
                $this->_postErrors[] = $this->l('Sandbox Signing Key is required.');	
			else if (!Tools::getValue('environment'))
				$this->_postErrors[] = $this->l('Environment Type is required.');
		}
	}

    /**
	 * Save/update the Razer Merchant Services configuration setting
	 * 
	 */
	protected function _postProcess() {
		if (isset($_POST['btnSubmit'])) {
			Configuration::updateValue('AXAIPAY_MERCHANT_ID', Tools::getValue('merchant_id'));
			Configuration::updateValue('AXAIPAY_SIGNING_KEY', Tools::getValue('signing_key'));
            Configuration::updateValue('AXAIPAY_SANDBOX_MERCHANT_ID', Tools::getValue('sandbox_merchant_id'));
            Configuration::updateValue('AXAIPAY_SANDBOX_SIGNING_KEY', Tools::getValue('sandbox_signing_key'));
            Configuration::updateValue('AXAIPAY_ENVIRONMENT', Tools::getValue('environment'));
		}
		$this->_html .= $this->displayConfirmation($this->l('Settings updated'));
	}

    /**
	 * Display notification after saving the Razer Merchant Services configuration setting
	 * 
	 */
	private function _displayAxaipay() {
		return $this->display(__FILE__, 'infos.tpl');
	}

    /**
	 * Display the Razer Merchant Services configuration setting. <call private method>
	 * 
	 * @return string
	 */
	public function getContent()
	{

		if (Tools::isSubmit('btnSubmit'))
		{
			$this->_postValidation();
			if (!count($this->_postErrors))
				$this->_postProcess();
			else
				foreach ($this->_postErrors as $err)
					$this->_html .= $this->displayError($err);
		}
		else
			$this->_html .= '<br />';


		$this->_html .= $this->_displayAxaipay();
		$this->_html .= $this->renderForm();

		return $this->_html;
	}


	

    // hookPaymentOptions()
    /**
     * Hook the payment form to the prestashop Payment method. Display in payment method selection
     * 
     * @param array $params
     * @return string
     */
    public function hookPaymentOptions($params) {
        if (!$this->active)
            return;
        if (!$this->checkCurrency($params['cart']))
            return;		
			
        $customer    = new Customer((int)$this->context->cart->id_customer);
        $merchant_id = "";
        $signing_key = "";
        $url = "";

        if (Configuration::get('AXAIPAY_ENVIRONMENT') == "Sandbox") {
            $merchant_id = Configuration::get('AXAIPAY_SANDBOX_MERCHANT_ID');
            $signing_key = Configuration::get('AXAIPAY_SANDBOX_SIGNING_KEY');
            $url = "https://staging.axaipay.my/gateway/v1/payment";
        }
        else {
            $merchant_id = Configuration::get('AXAIPAY_MERCHANT_ID');
            $signing_key = Configuration::get('AXAIPAY_SIGNING_KEY');
            $url = "https://secured.axaipay.my/gateway/v1/payment";
        }
        

        $orderid = $this->context->cart->id;

		$rand_orderid .= "PS_";
		$chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789abcdefghijklmnopqrstuvwxyz";  
		$size = strlen( $chars );    
		for( $i = 0; $i < 6; $i++ ) {  
			$rand_orderid .= $chars[ rand( 0, $size - 1 ) ];  
		} 
        $rand_orderid .= "_";

		$rand_orderid .= $orderid;

        $amount = $this->context->cart->getOrderTotal(true, Cart::BOTH);
		$amount = number_format($amount,2,".","");
        $bill_name = $customer->firstname." ".$customer->lastname;
        $bill_email = $customer->email;
        
        $country_obj =  new Country(intval($address->id_country));
        $country = $country_obj->iso_code;
        $country_name_obj = $country_obj->name;
        $country_name =  $country_name_obj[1];
        
        $add_obj = new Address(intval($params['cart']->id_address_invoice));
        $add = "";
        $add.="----------------------------------\nBilling Address\n----------------------------------\n";
        $add.= $add_obj->address1." ".$add_obj->address2." ".$add_obj->postcode." ".$add_obj->city." ".$country_name;
        $prod_obj = $params['cart']->getProducts();
        $size = sizeof($prod_obj);
        $prod = "";
        $prod.= "\n\n\n----------------------------------\nProduct(s) Info\n----------------------------------\n";
        for($i=0; $i<$size; $i++) {
            $prod.= $prod_obj[$i]['name']." x ".$prod_obj[$i]['cart_quantity'];
            $prod.= "\n";
        }
        $phone = $add_obj->phone_mobile;

        //$cart = new Cart(intval($orderid));

		$bill_desc = $add.$prod;
        
		// Signature for Axaipay verification
		$string_to_sign = $bill_email . // customerEmail
						$bill_name . // customerName
						$phone . // customerPhone
						$merchant_id . // mchtId
						$rand_orderid . // mchtTrxnId
						"Orange Juice" . // OrderDescription
						'https://'.$_SERVER['HTTP_HOST'].__PS_BASE_URI__.'index.php?fc=module&module=axaipay&controller=validation' . // redirectUrl
						$amount // txnAmount
						;
		$digest = hash_hmac('sha512', $string_to_sign, $signing_key, true);

        // $this->smarty->assign(array(
        //     'amount' => $amount,
        //     'orderid' => $orderid,
        //     'mp_merchant' => $mp_merchant,
        //     'bill_name' => $bill_name,
        //     'bill_email' => $bill_email,
        //     'bill_desc' => $bill_desc,
        //     'molpayUrl' => $url.'pay/'.$mp_merchant.'/',
        //     'currency' => $currency_code,
        //     'country' =>  $country,
        //     'vcode'   => $vcode,
        //     'returnurl' => 'http://'.$_SERVER['HTTP_HOST'].__PS_BASE_URI__.'index.php?fc=module&module=molpay&controller=validation',
        //     'this_path' => $this->_path
        // ));

        $this->smarty->assign(array(
			'url' => $url,
            'mchtId' => $merchant_id,
            'mchtTrxnId' => $rand_orderid,
            'signature' => base64_encode($digest),
            'txnAmount' => $amount,
            'customerEmail' => $bill_email,
            'customerName' => $bill_name,
            'customerPhone' => $phone,
            'orderDescription' => "Orange Juice",
            'redirectUrl' => 'https://'.$_SERVER['HTTP_HOST'].__PS_BASE_URI__.'index.php?fc=module&module=axaipay&controller=validation'
        ));

        $newOption = new PaymentOption();
        $newOption->setModuleName($this->name)
                    ->setCallToActionText($this->l('Axaipay'))
                    ->setAdditionalInformation($this->fetch('module:axaipay/views/templates/front/payment_infos.tpl'));

        return [$newOption];
    }

    // hookPaymentReturn()
	/**
     * Hook the payment return to the prestashop payment return method
     * 
     * @param array $params
     * @return string
     */
	public function hookPaymentReturn($params) {
		if (!$this->active)
			return;
		
		$state = $params['order']->getCurrentState();
		if ($state == Configuration::get('PS_OS_PAYMENT')) {
			$this->smarty->assign(array(
				'shop_name' => $this->context->shop->name,
				'status' => '11',
				'merchantID' => Configuration::get('AXAIPAY_MERCHANT_ID'),
				'id_order' => $params['order']->id
			));
			if(isset($params['order']->reference) && !empty($params['order']->reference))
					$this->smarty->assign('reference', $params['order']->reference);
		}		
		else if ($state == Configuration::get('PS_OS_PREPARATION')) {
			$this->smarty->assign(array(
					'shop_name' => $this->context->shop->name,
					'status' => '2',
					'merchantID' => Configuration::get('AXAIPAY_MERCHANT_ID'),
					'id_order' => $params['order']->id
			));
			if (isset($params['order']->reference) && !empty($params['order']->reference))
				$this->smarty->assign('reference', $params['order']->reference);
		}
		else if ($state == Configuration::get('PS_OS_ERROR')) {
			$this->smarty->assign(array(
					'shop_name' => $this->context->shop->name,
					'status' => '22',
					'merchantID' => Configuration::get('AXAIPAY_MERCHANT_ID'),
					'id_order' => $params['order']->id
			));
			if (isset($params['order']->reference) && !empty($params['order']->reference))
				$this->smarty->assign('reference', $params['order']->reference);
		}
		else
			$this->smarty->assign('status', 'other');
        
		return $this->fetch('module:axaipay/views/templates/hook/payment_return.tpl');
	}
    /**
	 * Check the currency
	 * 
	 * @param object $cart
	 * @return boolean
	 */
	public function checkCurrency($cart) {
		$currency_order = new Currency($cart->id_currency);
		$currencies_module = $this->getCurrency($cart->id_currency);

		if (is_array($currencies_module))
			foreach ($currencies_module as $currency_module)
				if ($currency_order->id == $currency_module['id_currency'])
					return true;
		return false;
	}

    public function renderForm()
	{
		$fields_form = array(
			'form' => array(
				'legend' => array(
					'title' => $this->l('Account details'),
					'icon' => 'icon-envelope'
				),
				'input' => array(
					array(
						'type' => 'text',
						'label' => $this->l('Merchant ID'),
						'name' => 'merchant_id',
						'required' => true
					),
					array(
						'type' => 'text',
						'label' => $this->l('Signing Key'),
						'name' => 'signing_key',
						'required' => true
					),
					array(
						'type' => 'text',
						'label' => $this->l('Sandbox Merchant ID'),
						'name' => 'sandbox_merchant_id',
						'required' => true
					),
                    array(
						'type' => 'text',
						'label' => $this->l('Sandbox Signing Key'),
						'name' => 'sandbox_signing_key',
						'required' => true
					),
					array(
						'type' => 'select',
						'label' => $this->l('Environment Type'),
						'desc' => $this->l('Default will be Sandbox'),
						'name' => 'environment',
						'required' => true,
						'options' => array(
							'query' => array(
										array('environment' => 'Sandbox','name' => 'Sandbox'),
										array('environment' => 'Production','name' => 'Production'),
										),                           
							'id' => 'environment',
							'name' => 'name'
						)
					)
				),
				'submit' => array(
					'title' => $this->l('Save'),
				)
			),
		);


		$helper = new HelperForm();
		$helper->show_toolbar = false;
		$helper->table = $this->table;
		$lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
		$helper->default_form_language = $lang->id;
		$helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
		$this->fields_form = array();
		$helper->id = (int)Tools::getValue('id_carrier');
		$helper->identifier = $this->identifier;
		$helper->submit_action = 'btnSubmit';
		$helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
		$helper->token = Tools::getAdminTokenLite('AdminModules');
		$helper->tpl_vars = array(
			'fields_value' => $this->getConfigFieldsValues(),
			'languages' => $this->context->controller->getLanguages(),
			'id_language' => $this->context->language->id
		);

		return $helper->generateForm(array($fields_form));
	}

    public function getConfigFieldsValues()
	{
		$AXAIPAY_ENVIRONMENT = Configuration::get('AXAIPAY_ENVIRONMENT');
		if( empty($AXAIPAY_ENVIRONMENT) )
			$AXAIPAY_ENVIRONMENT = "Sandbox";
		
		
		$result = array(
			'merchant_id'	=> Tools::getValue('merchant_id', Configuration::get('AXAIPAY_MERCHANT_ID')),
			'signing_key' => Tools::getValue('signing_key', Configuration::get('AXAIPAY_SIGNING_KEY')),
            'sandbox_merchant_id'	=> Tools::getValue('sandbox_merchant_id', Configuration::get('AXAIPAY_SANDBOX_MERCHANT_ID')),
			'sandbox_signing_key' => Tools::getValue('sandbox_signing_key', Configuration::get('AXAIPAY_SANDBOX_SIGNING_KEY')),
			'environment' 	=> Tools::getValue('environment', $AXAIPAY_ENVIRONMENT),
		);

		return $result;
	}

}

?>